// Copyright (C) 2002-2003 Chad C. Yates cyates@uidaho.edu
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// 
// As a special exception to the GNU General Public License, permission is 
// granted for additional uses of the text contained in its release 
// of Common C++.
// 
// The exception is that, if you link the Common C++ library with other
// files to produce an executable, this does not by itself cause the
// resulting executable to be covered by the GNU General Public License.
// Your use of that executable is in no way restricted on account of
// linking the Common C++ library code into it.
//
// This exception does not however invalidate any other reasons why
// the executable file might be covered by the GNU General Public License.
//
// This exception applies only to the code released under the 
// name Common C++.  If you copy code from other releases into a copy of
// Common C++, as the General Public License permits, the exception does
// not apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
// 
// If you write modifications of your own for Common C++, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice.

#include "Test_Digest.h"

// register the test suites
CPPUNIT_TEST_SUITE_REGISTRATION(ChecksumDigestTest);
CPPUNIT_TEST_SUITE_REGISTRATION(CRC16DigestTest);
CPPUNIT_TEST_SUITE_REGISTRATION(CRC32DigestTest);
CPPUNIT_TEST_SUITE_REGISTRATION(MD5DigestTest);
CPPUNIT_TEST_SUITE_REGISTRATION(SHA1DigestTest);
CPPUNIT_TEST_SUITE_REGISTRATION(SHA256DigestTest);

template<class DigestTypeClass>
void GenericDigestTests<DigestTypeClass>::testTypicalUsage()
{
  // run through the tests in a typical usage order and check last digest
  digest << getMsg1();
  digest.initDigest();
  digest << getLongMsg1();
  digestOutput << digest;
  CPPUNIT_ASSERT_EQUAL_MESSAGE(
    "Hashed Message was '" + getLongMsg1() + "'",
    getLongMsg1_Digest(), // expected
    digestOutput.str()  // actual
  );
}

template<class DigestTypeClass>
void GenericDigestTests<DigestTypeClass>::testConstruction()
{
  digestOutput << digest;
  CPPUNIT_ASSERT_EQUAL_MESSAGE(
    "Digest just after initialization",
    getEmptyMsg_Digest(), // expected
    digestOutput.str()  // actual
  );
}

template<class DigestTypeClass>
void GenericDigestTests<DigestTypeClass>::testEmptyData()
{
	digest << getEmptyMsg();
  digestOutput << digest;
  CPPUNIT_ASSERT_EQUAL_MESSAGE(
    "Hashed Message was '" + getEmptyMsg() + "'",
    getEmptyMsg_Digest(), // expected
    digestOutput.str()  // actual
  );
}
 
template<class DigestTypeClass>
void GenericDigestTests<DigestTypeClass>::testSmallMessage()
{
	digest << getMsg1();
  digestOutput << digest;
  CPPUNIT_ASSERT_EQUAL_MESSAGE(
    "Hashed Message was '" + getMsg1() + "'",
    getMsg1_Digest(), // expected
    digestOutput.str()  // actual
  );
}

template<class DigestTypeClass>
void GenericDigestTests<DigestTypeClass>::testLargeMessage()
{  
	digest << getLongMsg1();
  digestOutput << digest;
  CPPUNIT_ASSERT_EQUAL_MESSAGE(
    "Hashed Message was '" + getLongMsg1() + "'",
    getLongMsg1_Digest(),  // expected
    digestOutput.str()       // actual
  );
}

template<class DigestTypeClass>
void GenericDigestTests<DigestTypeClass>::testBinaryData()
{  
	digest.putDigest(getBinaryData1(), 256);
  digestOutput << digest;
  CPPUNIT_ASSERT_EQUAL_MESSAGE(
    "Data was binary, cannot display",
    getBinaryData1_Digest(),  // expected
    digestOutput.str()       // actual
  );
}

template<class DigestTypeClass>
void GenericDigestTests<DigestTypeClass>::testReInitialization()
{
  // calculate a digest before re-initializing
	digest << getMsg1();
  digestOutput << digest;

  // after re initializing
  digest.initDigest();
  digestOutput.str(""); // clear the stringstream
  digestOutput << digest;
  CPPUNIT_ASSERT_EQUAL_MESSAGE(
    "Digest after re-initialization",
    getEmptyMsg_Digest(), // expected
    digestOutput.str()  // actual
  );
}

template<class DigestTypeClass>
void GenericDigestTests<DigestTypeClass>::testPiecewise()
{
  for(unsigned i = 0; i < getLongMsg1().length(); ++i)
    digest.putDigest((const unsigned char *)getLongMsg1().substr(i, 1).c_str(), 1);
  digestOutput << digest;
  CPPUNIT_ASSERT_EQUAL_MESSAGE(
    "Hashed Message was '" + getLongMsg1() + "'",
    getLongMsg1_Digest(), // expected
    digestOutput.str()  // actual
  );
}
